package com.chelvc.framework.wechat.support;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.wechat.WechatPayRequest;
import com.chelvc.framework.wechat.WechatPaymentCallback;
import com.chelvc.framework.wechat.WechatPaymentHandler;
import com.chelvc.framework.wechat.WechatUnifiedOrder;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 微信支付处理器委派实现
 *
 * @author Woody
 * @date 2024/6/19
 */
@Slf4j
public class DelegatingWechatPaymentHandler implements WechatPaymentHandler {
    private final List<WechatPaymentHandler> handlers;

    public DelegatingWechatPaymentHandler(@NonNull List<WechatPaymentHandler> handlers) {
        AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
        this.handlers = Lists.newArrayList(handlers);
    }

    @Override
    public boolean supports(@NonNull String name) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String sign(@NonNull String name, @NonNull Map<String, String> parameters) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                return handler.sign(name, parameters);
            }
        }
        throw new IllegalArgumentException("Wechat payment handler not found: " + name);
    }

    @Override
    public WechatPaymentCallback callback(@NonNull String name, @NonNull String content) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                try {
                    WechatPaymentCallback callback = handler.callback(name, content);
                    if (Objects.nonNull(callback)) {
                        return callback;
                    }
                } catch (Exception e) {
                    log.error("Wechat payment callback handle failed: {}", content, e);
                }
            }
        }
        return null;
    }

    @Override
    public WechatUnifiedOrder unifiedorder(@NonNull String name, @NonNull WechatPayRequest request) {
        for (WechatPaymentHandler handler : this.handlers) {
            if (handler.supports(name)) {
                return handler.unifiedorder(name, request);
            }
        }
        throw new IllegalArgumentException("Wechat payment handler not found: " + name);
    }
}
